import 'package:animated_infinite_scroll_pagination/src/ui/pagination_error.dart';
import '../models/pagination_model.dart';
import 'pagination_scroll_view.dart';
import '../viewmodels/pagination_viewmodel.dart';
import 'package:flutter/material.dart';
import 'package:flutterx_live_data/flutterx_live_data.dart';
import 'package:implicitly_animated_reorderable_list/implicitly_animated_reorderable_list.dart';
import 'package:implicitly_animated_reorderable_list/transitions.dart';
import 'pagination_loader.dart';

class AnimatedPaginationScrollView<T extends Object> extends StatelessWidget {
  final PaginationViewModel<T> viewModel;
  final ScrollPhysics? physics;
  final Widget Function(T) itemBuilder;
  final Widget? topWidget;
  final Widget? loadingWidget;
  final Widget? errorWidget;
  final Widget? footerLoadingWidget;

  const AnimatedPaginationScrollView({
    required this.viewModel,
    required this.itemBuilder,
    this.topWidget,
    this.footerLoadingWidget,
    this.loadingWidget,
    this.errorWidget,
    this.physics,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return PaginationScrollView(
      key: UniqueKey(),
      physics: physics,
      paginationParams: viewModel.params,
      loadingWidget: loadingWidget,
      child: Column(
        children: [
          if (topWidget != null) topWidget!,
          LiveDataBuilder<PaginationEquatable<T>>(
            key: UniqueKey(),
            data: viewModel.params.itemsList,
            builder: (context, list) {
              return Material(
                color: Colors.transparent,
                elevation: 0,
                child: ImplicitlyAnimatedList<PaginationModel<T>>(
                  shrinkWrap: true,
                  physics: const NeverScrollableScrollPhysics(),
                  items: list.items,
                  areItemsTheSame: (a, b) => viewModel.areItemsTheSame(a.item, b.item),
                  itemBuilder: (context, animation, item, i) {
                    return SizeFadeTransition(
                      animation: animation,
                      child: itemBuilder.call(item.item),
                    );
                  },
                  updateItemBuilder: (context, animation, item) {
                    return FadeTransition(
                      opacity: animation,
                      child: itemBuilder.call(item.item),
                    );
                  },
                ),
              );
            },
          ),
          PaginationError(paginationParams: viewModel.params, errorWidget: errorWidget),
          PaginationLoader(paginationParams: viewModel.params, loadingWidget: footerLoadingWidget),
        ],
      ),
    );
  }
}
